/**********************************************************************/
/* $Id:  $
 **********************************************************************/
/*
 *  (C) Copyright SAP AG, Walldorf 2004, All Rights reserved
 *
 * SAP AG DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL SAP AG BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 */

/*header--------------------------------------------------------------*/
/*                                                                    */
/* Description:                                                       */
/* ============                                                       */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*  Author:        Markus Strehle                                     */
/*  Reviewer:                                                         */
/*  Version:       0.90                                               */
/*                                                                    */
/*  Created:                                                          */
/*    20 June 2005  Markus Strehle                                    */
/*                                                                    */
/*  Modified:                                                         */
/*    15 Aug  2008  Markus Strehle                                    */
/*                  (many changes)                                    */
/*                                                                    */
/*  Remarks:                                                          */
/*      Modifications are done for NW-VSI Version 1.00                */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*                                                                    */
/* --- public functions ----------------------------------------------*/
/*                                                                    */
/*      VsaStartup                                                    */
/*      VsaInit                                                       */
/*      VsaGetConfig                                                  */
/*      VsaScan                                                       */
/*      VsaReleaseScan                                                */
/*      VsaEnd                                                        */
/*      VsaCleanup                                                    */
/*                                                                    */
/**********************************************************************/
#ifndef VSCLAM_H
#define VSCLAM_H

#ifdef __cplusplus
extern "C"
{
#endif

/*--------------------------------------------------------------------*/
/* VSA example defines                                                */
/*--------------------------------------------------------------------*/
/* this adapter provides NW-VSI version 1 */
#define VSI_VERSION            1

/* adapter constants in VSA_CONFIG        */
#define VSA_ADAPTER_ID         "ClavAV"
#define VSA_ADAPTER_MAJVER     1
#define VSA_ADAPTER_MINVER     0
#define VSA_VENDORINFO         "OpenSource adapter, see http://clamsap.sourceforge.net/"
#define VSA_VENDORINFO_LN      55
#define VSA_ADAPTERNAME        "VSCLAM Adapter for NW-VSI     "
#define VSA_ADAPTERNAME_LN     30

#define VSA_VERSION_STRING     "ClamSAP VSA for libclamav"

/* we can not access to the engine, so we know 1 driver/definition */
#define CLEANUP(x)          { rc = x; goto cleanup; }
/* default for VSA_CONFIG: the current directory*/
#ifdef _WIN32
#define DIR_SEP             "\\"
#define DRIVER_DIRECTORY    "."
#define DRIVER_DIRECTORY_LN 1
#else
#define DIR_SEP             "/"
#define DRIVER_DIRECTORY    "."
#define DRIVER_DIRECTORY_LN 1
#endif

#define CLAMAV_DRIVERS      "main.cvd,daily.cvd"
#define MAX_DRIVERS         2

#define CLAMAV_DRIVERS_LN   18

#define ENGINE_DATA(x,y,z) \
    _utc_date.tm_mon = y-1;  \
    _utc_date.tm_mday= x;    \
    _utc_date.tm_year= z-1900;

/* define exported sysmbols for zOS */
#ifdef SAPonOS390
#pragma export(VsaStartup)
#pragma export(VsaGetConfig)
#pragma export(VsaInit)
#pragma export(VsaScan)
#pragma export(VsaReleaseScan)
#pragma export(VsaEnd)
#pragma export(VsaCleanup)
#endif
/* structs for VsaGetConfig returning own configuration */
typedef struct {
    VS_INITPARAM_T     tCode;
    VS_PARAMTYPE_T     tType;
    size_t             lLength;
    VSA_PARAMVALUE     pvValue;
} MY_INITPARAMS;

typedef struct  {
    VS_OPTPARAM_T      tCode;
    VS_PARAMTYPE_T     tType;
    size_t             lLength;
    VSA_PARAMVALUE     pvValue;
} MY_OPTPARAMS;

/* structure for transporting our usrdata + our 
 * function pointer + virus_info 
 */
struct usrdata {
    VSA_RC          vsa_rc;
    UInt            uiJobID;
    UInt            uiMsgFlags;
    size_t          lObjectSize;
    VSA_EVENTCBFP   pvFncptr;
    void           *pvUsrdata;
    Bool            bScanBestEffort;
    Bool            bScanAllFiles;
    VSA_SCANINFO   *pScanInfo;
    VS_MESSAGE_T    tMsg_rc;
};
typedef struct usrdata USRDATA, *PUSRDATA, **PPUSRDATA;

struct initdata {
   PVSA_INITPARAM   initdirectory;
   PVSA_INITPARAM   drivers;
   PVSA_INITPARAM   tmpdir;
};
typedef struct initdata INITDATA, *PINITDATA;

/* helper macros */
#define VSAddINITParameter(pl, i, c, a, b, s) \
{       pl[i].struct_size    = sizeof(VSA_INITPARAM); \
        pl[i].tCode          = c; \
        pl[i].tType          = a; \
        pl[i].lLength        = b; \
        pl[i].pvValue        = s; \
        i++; \
}
#define VSAddOPTParameter(pl, i, c, a, b, s) \
{       pl[i].struct_size    = sizeof(VSA_OPTPARAM); \
        pl[i].tCode          = c; \
        pl[i].tType          = a; \
        pl[i].lLength        = b; \
        pl[i].pvValue        = s; \
        i++; \
}

#define CB_FUNC(msg, udata)                         \
    ((cfunc != NULL) && (uiMsgFlag & msg)) ?        \
       cfunc ( heng, msg, (VSA_PARAM)udata, pvUsrdata ) : VS_CB_OK;

#define SET_VSA_RC(_vsa_rc_) \
{   if (usrdata.vsa_rc == VSA_OK || usrdata.vsa_rc > _vsa_rc_) \
    usrdata.vsa_rc = _vsa_rc_; }

#define SETSTRING( buf, txt )                                           \
{   if (txt != NULL && (len=strlen((const char*)txt)) > (size_t)0)      \
    {   buf = (PChar)malloc(len+1);                                     \
    if (buf == NULL)                                                    \
        CLEANUP(VSA_E_NO_SPACE);                                        \
    /* CCQ_OFF */ memcpy(buf,txt,len); /*CCQ_ON */                      \
    buf[len] = 0; } /* ensure zero termination */                       \
    else                                                                \
        buf = NULL;                                                     \
}

#define SETERRORTEXT    SETSTRING

#ifdef __cplusplus
}
#endif

#endif /* VSCLAM_H */
